//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// June 20, 2000
//
#ifndef IPPVEXTRACTORIMPLEMENTATION_H
#define IPPVEXTRACTORIMPLEMENTATION_H

#include "IppvExtractor.h"
#include <string>
#include <map>

class IppvExtractorCommon: virtual public IppvExtractor {
 public:
  IppvExtractorCommon(const Xmm::Instrument instrument,
		      const string& directory = ".",
		      const string& label = "");
  virtual ~IppvExtractorCommon() {}
  virtual unsigned ippvs() const;
  virtual void write(ostream& o,
		     const STime& start,
		     const STime& stop) const;
  virtual void document(ostream& o) const;
  virtual Xmm::Instrument instrument() const;
 protected:
  string _dir;
  Xmm::Instrument _instrument;
  mutable map<Xmm::DataMode, vector<Ippv *> > _ippvs;
  const string _label;
};  

class Rgs1IppvExtractor: public IppvExtractorCommon {
 public:
  Rgs1IppvExtractor(const string& directory = ".", const string& label = "");
  ~Rgs1IppvExtractor() {}
};  

class Rgs2IppvExtractor: public IppvExtractorCommon {
 public:
  Rgs2IppvExtractor(const string& directory = ".", const string& label = "");
  ~Rgs2IppvExtractor() {}
};  

class Emos1IppvExtractor: public IppvExtractorCommon {
 public:
  Emos1IppvExtractor(const string& directory = ".", const string& label = "");
  ~Emos1IppvExtractor() {}
};  

class Emos2IppvExtractor: public IppvExtractorCommon {
 public:
  Emos2IppvExtractor(const string& directory = ".", const string& label = "");
  ~Emos2IppvExtractor() {}
};  

class EpnIppvExtractor: public IppvExtractorCommon {
 public:
  EpnIppvExtractor(const string& directory = ".", const string& label = "");
  ~EpnIppvExtractor() {}
};  

class OmIppvExtractor: public IppvExtractorCommon {
 public:
  OmIppvExtractor(const string& directory = ".", const string& label = "");
  ~OmIppvExtractor() {}
};  

#endif
