//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// June 20, 2000
//
#ifndef IPPVIMPLEMENTATION_H
#define IPPVIMPLEMENTATION_H

#include <map>
#include <vector>
#include <DataFilter.h>

#include <Ippv.h>
#include <SasDalUtils.h>

#include <IppvTranslator.h>

// Two classes for Periodic and Non Periodic IPPVs, as the algorithms
// they have to applyt to find the data are different.

// Shared implementation in IppvCommon.

template<class T,
  class S,
  S * (Data::*data)() const
  >
class IppvCommon : public virtual Ippv {

 public:
  IppvCommon(const string& name,
	     const string& colname,
	     const string& label,
	     const bool& singlevalue);
  
  IppvCommon(const string& name,
	     const string& colname,
	     const string& label,
	     const string& expression,
	     const bool& singlevalue);
  
  virtual ~IppvCommon();
  
  void fetch(const TimeOrderedTable *tab,
	     const STime& start,
	     const STime& stop) throw(IppvException);
  void print (ostream& os, const bool verbose) const;
  void notAvailable(ostream& os) const;
  const string& name() const;
  void addTranslator(IppvTranslator<T> *t);

  void setValue(const Column *column,
		const vector<DataFilter::RowRange>& r) throw(IppvException);

  virtual unsigned lower_row(const TimeOrderedTable *tab,
			     const STime& t) const = 0;
  virtual unsigned upper_row(const TimeOrderedTable *tab,
			     const STime& t) const = 0;

 protected:
  string _name, _colname, _expression, _label;
  const bool _filter;
  const bool _singlevalue;
  T _value;
  bool _has_translator;
  IppvTranslator<T> *_trans;

};  

template<class T,
  class S,
  S * (Data::*data)() const
  >
class PeriodicIppv : public virtual Ippv,
		     public virtual IppvCommon<T,S,data> {

public:
  PeriodicIppv(const string& name,
	       const string& colname,
	       const string& label,
	       const bool& singlevalue);
  
  PeriodicIppv(const string& name,
	       const string& colname,
	       const string& label,
	       const string& expression,
	       const bool& singlevalue);
  
  //  ~PeriodicIppV();
/*  private: */


  unsigned lower_row(const TimeOrderedTable *tab,
		     const STime& t) const;
  unsigned upper_row(const TimeOrderedTable *tab,
		     const STime& t) const;

};

template<class T,
  class S,
  S * (Data::*data)() const
  >
class NonPeriodicIppv : public virtual Ippv,
			public virtual IppvCommon<T,S,data> {

public:
  NonPeriodicIppv(const string& name,
		  const string& colname,
		  const string& label,
		  const bool& singlevalue);
  
  NonPeriodicIppv(const string& name,
		  const string& colname,
		  const string& label,
		  const string& expression,
		  const bool& singlevalue);
  
  //  ~NonPeriodicIppV();

/*  private: */

  unsigned lower_row(const TimeOrderedTable *tab,
		     const STime& t) const;
  unsigned upper_row(const TimeOrderedTable *tab,
		     const STime& t) const;

};

#endif
